-- Title: Skill Helper
-- Author: JerichoHM
-- Maintainer: LownIgnitus
-- Version: 3.1.2
-- Desc: A simple addon for tracking and using skills

-- GLOBALS
local SKILLCAP = 700
local y = -15
local imgFolder = "Interface\\ICONS\\"
local shBarBG = { bgFile = "Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar", edgeFile = nil, tile = false, tileSize = 32, edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0}}
SLASH_SKILLHELPER1 = '/SHelper' or '/shelper'
local icon
if shLock == nil then
	shLock = true
end

--Version Information
local Author = "|cffffcc00JerichoHM|r"
local Maintainer = "|cffDA70D6LownIgnitus|r"
local Title = "|cff00ff00Skill Helper|r"
local Core = "|cffFF45003|r"
local Revision = "|cffFF45001|r"
local Build = "|cffFF45002|r"

-- Skills Table
local skills = {}

-- Frames Table
local bars = {}

function shDrawBar(name, texture, rank, rankModifier, maxRank, numSpells, y)
	local bar = bars[name]
	if not bar then --if bar doesn't exist
		-- The Skill bar
		bar = CreateFrame("StatusBar", nil, shFrame, nil)
		bar:SetFrameStrata("MEDIUM")
		bar:SetPoint("TOPLEFT", shFrame, "TOPLEFT", 6, y-4)
		bar:SetSize(160,18)
		bar:SetBackdrop(shBarBG)
		bar:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		bar:SetStatusBarTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar")
		bar:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		bar:SetMinMaxValues(0,1)
		bar:SetValue((rank + rankModifier)/maxRank)

		-- The text on the Skill Bar
		local barTxt = bar:CreateFontString(nil, "ARTWORK");
		isValid = barTxt:SetFont("Fonts\\FRIZQT__.TTF", 10);
		barTxt:SetPoint("CENTER", bar, "CENTER", 0, 1);
		bar.text = barTxt

		-- The first button on the skills action bar
		local barBtn1 = CreateFrame("Button", nil, bar, "SecureActionButtonTemplate");
		barBtn1:SetFrameStrata("MEDIUM");
		barBtn1:SetPoint("TOP", bar, "TOPRIGHT", 9, 0);
		barBtn1:SetSize(18,18);
		barBtn1:EnableMouse(true);
		barBtn1:SetAttribute("type", "spell");
		if name == "Mining" then
			barBtn1:SetAttribute("spell", "Smelting");
			barBtn1BG = { bgFile = imgFolder .. "spell_fire_flameblades", edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}};
			barBtn1:SetBackdrop(barBtn1BG);
			barBtn1:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_TOP"); GameTooltip:ClearLines(); GameTooltip:AddLine("Smelting"); GameTooltip:Show(); end)
			barBtn1:SetScript("OnLeave", function(self) GameTooltip:Hide(); end)
			bar.btn1 = barBtn1
		else
			barBtn1:SetAttribute("spell", name);
--			imgFolder .. shGetButton(name, 1)
			barBtn1BG = { bgFile = texture, edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}};
			barBtn1:SetBackdrop(barBtn1BG);
			barBtn1:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_TOP"); GameTooltip:ClearLines(); GameTooltip:AddLine(name); GameTooltip:Show(); end)
			barBtn1:SetScript("OnLeave", function(self) GameTooltip:Hide(); end)
			bar.btn1 = barBtn1
		end

		-- The second button on the skills action bar if applicable
		if numSpells == 2 then
--			print("Button 2" .. name)
			if name ~= "Skinning" and name ~= "Tailoring" and name ~= "Alchemy" and name ~= "Engineering" and name ~= "Blacksmithing" and name ~= "Leatherworking" and name ~= "Mining" then
				local barBtn2 = CreateFrame("Button", nil, barBtn1, "SecureActionButtonTemplate");
				barBtn2:SetFrameStrata("MEDIUM");
				barBtn2:SetPoint("TOP", barBtn1, "TOPRIGHT", 10, 0);
				barBtn2:SetSize(18,18);
				barBtn2:EnableMouse(true);
				barBtn2:SetAttribute("type", "spell");
				barBtn2:SetAttribute("spell", shGetSpell(name, 2));
				barBtn2BG = { bgFile = imgFolder .. shGetButton(name, 2) .. ".BLP", edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}};
				barBtn2:SetBackdrop(barBtn2BG);
				barBtn2:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_TOP"); GameTooltip:ClearLines(); GameTooltip:AddLine(shGetSpell(name, 2)); GameTooltip:Show(); end)
				barBtn2:SetScript("OnLeave", function(self) GameTooltip:Hide(); end)
				bar.btn2 = barBtn2
			end
		end

		-- Text formatting for the text on the Skill Bar
		bar.text:SetFormattedText("|cFFFFFF00%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)

		-- Add to the array
		bars[name] = bar
	else --if bar exists
		bar:ClearAllPoints()
	end

	-- if the rank is in trainable range and not skill cap then turn bar red
	if	rank >= maxRank - 25 and maxRank < SKILLCAP then
		bar:SetPoint("TOPLEFT", shFrame, "TOPLEFT", 6, y-4)
		bar:SetStatusBarColor(0.7, 0.0, 0.0, 1.0)
		bar:SetBackdropColor(0.7, 0.0, 0.0, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)

		bar.text:SetFormattedText("|cFFFF0000%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)
	elseif rank == maxRank then -- set maxed skill level bar royal blue
		bar:SetPoint("TOPLEFT", shFrame, "TOPLEFT", 6, y-4)
		bar:SetStatusBarColor(65/255, 105/255, 255/255, 1)
		bar:SetBackdropColor(65/255, 105/255, 255/255, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)

		bar.text:SetFormattedText("|cFF00FF00%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)
	else -- otherwise keep bar green
		bar:SetPoint("TOPLEFT", shFrame, "TOPLEFT", 6, y-4)
		bar:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		bar:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)

		bar.text:SetFormattedText("|cFFFFFF00%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)
	end

	shFrame:SetHeight((y - 27)* -1)
end

function shLinksButtons(name, y)
	linkBtn = CreateFrame("Button", nil, shLinksFrame, nil)
	linkBtn:SetFrameStrata("MEDIUM")
	linkBtn:SetPoint("TOPRIGHT", shLinksFrame, "TOPRIGHT", -5, y-4)
	linkBtn:SetSize(40, 18)
	linkBtn:EnableMouse(true)
	linkBtnBG = { edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 32, edgeSize = 16, insets = {left = 3, right = 3, top = 3, bottom = 3}};
	linkBtn:SetBackdrop(linkBtnBG)
	linkBtn:SetScript("OnClick", function(self)
		local link = select(2, GetSpellLink(name))
--		print(link)
		if link == nil then ChatFrame1:AddMessage("Could not get link for profession.") return end

		local activeEditBox = ChatEdit_GetActiveWindow()
		if MacroFrameText and MacroFrameText:IsShown() and MacroFrameText:HasFocus() then
			local text = MacroFrameText:GetText() .. link
			if strlenutf8(text) <= 255 then
				MacroFrameText:Insert(link)
			end
		elseif activeEditBox then
			ChatEdit_InsertLink(link)
		end
	end)
	linkBtn:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_TOP"); GameTooltip:ClearLines(); GameTooltip:AddLine(name .. " Link"); GameTooltip:Show(); end)
	linkBtn:SetScript("OnLeave", function(self) GameTooltip:Hide(); end)

	local linkTxt = linkBtn:CreateFontString(nil, "ARTWORK");
	isValid = linkTxt:SetFont("Fonts\\FRIZQT__.TTF", 14);
	linkTxt:SetPoint("CENTER", linkBtn, "CENTER", 0, 0);
	linkBtn.text = linkTxt
	linkBtn.text:SetText("|cFFFFFF00Link|r")
	
	shLinksFrame:SetHeight((y - 27)* -1)
end

function shUpdateData()
	-- Capture the SkillIndes for the players Professions
	local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()
--	print(prof1 .. ", " .. prof2 .. ", " .. archaeology .. ", " .. fishing .. ", " .. cooking .. ", " .. firstAid)

	-- Build an array of the Professions
	local profs = {prof1, prof2, archaeology, fishing, cooking, firstAid}

	for k, v in pairs(profs) do
		-- Fetch the details for each Profession
		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset = GetProfessionInfo(v)
--		print(name .. ", " .. texture .. ", " .. rank .. ", " .. maxRank .. ", " .. numSpells .. ", " .. spelloffset .. ", " .. skillLine .. ", " .. rankModifier .. ", " .. specializationIndex .. ", " .. specializationOffset)
		shDrawBar(name, texture, rank, rankModifier, maxRank, numSpells, y)
		shLinksButtons(name, y)
		y = y - 18

		table.insert( skills, name)
	end

	shCleanUp()

	y = -15
end

function shCleanUp()
	local match = false
	local clnBar

	for k, d in pairs(bars) do -- cycle bar names
		for i, v in ipairs(skills) do -- cycle skill nams
			if k == v then -- if the names match set true
				match = true
				break
			else -- otherwise false
				match = false
			end
		end

		-- store bar to local variable
		clnBar = bars[k]

		if match == false then -- If bar and skill don't match hide bar and set to false
			clnBar:Hide()
			match = false
		elseif match == true then -- If bar and skill match show
			clnBar:Show()
		end
	end

	while ((# skills) > 0) do -- cycle through number of skills
		table.remove( skills, 1 ) -- remove skills from the table
	end
end

function shOnEvent(self, event, ...)
	--Title Frames
	shFrame.title.text:SetFormattedText("%s %s.%s.%s", Title, Core, Revision, Build)
	shLinksFrame.title.text:SetText("Links")

	local addon = select(1, ...);
	--print(addon)

	if event == "SKILL_LINES_CHANGED" then
		--print("Skill Change")
		if not InCombatLockdown() then
			shUpdateData()
		end
	elseif event == "ADDON_LOADED" then
		--print("Addon Loaded")
		if addon == "SkillHelper" then
			--print("Addon SH")
			if shAutohide == nil then
				shAutohide = false
			end

			if shLock == true then
				shFrame:EnableMouse(true)
			elseif shLock == false then
				shFrame:EnableMouse(false)
			end
			
			if shHidden == nil then
				shHidden = false
				shFrame:Show()
			elseif shHidden == false then
				shFrame:Show()
			elseif shHidden == true then
				shFrame:Hide()
			end			

			if shlHidden == nil then
				shlHidden = true
				shLinksFrame:Hide()
			elseif shlHidden == false then
				shLinksFrame:Show()
			elseif shlHidden == true then
				shLinksFrame:Hide()
			end

			if shDB == nil then
				shDB = { hide = false }
			end

			shMiniMap();
		
			if shDB.hide then
				icon:Hide("SkillHelper")
				shDB.hide = true
			else
				icon:Show("SkillHelper")
				shDB.hide = false
			end
			return;
		end
	end
end

function shGetButton(name, spellNum)
	local sName, rank, icon, castTime, minRange, maxRange = GetSpellInfo(name)
--	print(sName .. ", " .. rank .. ", " .. icon .. ", " .. castTime .. ", " .. minRange .. ", " .. maxRange)
	if name == "Archaeology" then
		if spellNum == 2 then
			return "inv_misc_shovel_01"
		end
	elseif name == "Cooking" then
		if spellNum == 2 then
			return "spell_fire_fire"
		end
	elseif name == "Inscription" then
		if spellNum == 2 then
			return "ability_miling"
		end
	elseif name == "Jewelcrafting" then
		if spellNum == 2 then
			return "inv_misc_gem_bloodgem_01"
		end
	elseif name == "Enchanting" then
		if spellNum == 2 then
			return "INV_Enchant_Disenchant"
		end
	end
end

function shGetSpell(name, spellNum)
	if name == "Archaeology" then
		if spellNum == 2 then
			return "Survey"
		end
	elseif name == "Cooking" then
		if spellNum == 2 then
			return "Cooking Fire"
		end
	elseif name == "Herbalism" then
		if spellNum == 2 then
			return "Lifeblood"
		end
	elseif name == "Inscription" then
		if spellNum == 2 then
			return "Milling"
		end
	elseif name == "Jewelcrafting" then
		if spellNum == 2 then
			return "Prospecting"
		end
	elseif name == "Enchanting" then
		if spellNum == 2 then
			return "Disenchant"
		end
	end
end

function SlashCmdList.SKILLHELPER(msg)
	if msg == "toggle" then
		shToggle()
	elseif msg == "lock" then
		shLocker()
	elseif msg == "links" then
		shLinks()
	elseif msg == "reset" then
		shReset()
	elseif msg == "info" then
		shInfo()
	elseif msg == "mmtoggle" then
		shMMToggle()
	else
		print("|cff71C671Skill Helper Slash Commands|r")
		print("|cff71C671type /SHelper toggle to toggle the addon hidden state|r")
		print("|cff71C671type /SHelper lock to toggle locking|r")
		print("|cff71C671type /SHelper links to toggle the Skill Links frame|r")
		print("|cff71C671type /SHelper reset to reset to default position|r")
		print("|cff71C671type /SHelper mmtoggle to toggle the minimap button on/off|r")
		print("|cff71C671tpye /SHelper info to view current build information|r")
	end
end

function shToggle()
	-- true for it is hidden and false for it isn't hidden
	if shHidden == false then
		shFrame:Hide()
		print("Skill Helper |cffff0000hidden|r!")
		shHidden = true
	elseif shHidden == true then
		shFrame:Show()
		print("Skill Helper |cff00ff00visible|r!")
		shHidden = false
	end
end

function shLinks()
	-- functions on same pcinciple as shToggle
	if shlHidden == false then
		shLinksFrame:Hide()
		print("Links frame |cffff0000hidden|r!")
		shlHidden = true
	elseif shlHidden == true then
		shLinksFrame:Show()
		print("Links frame |cff00ff00visible|r!")
		shlHidden = false
	end
end

function shLocker()
	-- Remember shLock is backwards. true for unlocked and false for locked
	if shLock == true then
		shLock = false
		shFrame:EnableMouse(shLock)
--		lockBtn:SetNormalTexture(imgFolder .. "LockButton-Locked-Up")
--		lockBtn:SetPushedTexture(imgFolder .. "LockButton-Locked-Down")
		print("Skill Helper |cffff0000locked|r!")
	elseif shLock == false then
		shLock = true
		shFrame:EnableMouse(shLock)
--		lockBtn:SetNormalTexture(imgFolder .. "LockButton-Unlocked-Up")
--		lockBtn:SetPushedTexture(imgFolder .. "LockButton-Unlocked-Down")
		print("Skill Helper |cff00ff00unlocked|r!")
	end
end

function shReset()
	shFrame:SetPoint("CENTER", UIParent, "CENTER")
	shLock = true
	shFrame:EnableMouse(shLock)
	shFrame:Show()
	shHidden = false
	print("Skill Helper Reset")
end

function shInfo()
	print(Title .. " " .. Core .. "." .. Revision .. "." .. Build)
	print("Author: " .. Author .. " / Maintainer: " .. Maintainer)
end

function shMMToggle()
	if shDB.hide then
		icon:Show("SkillHelper")
		shDB.hide = false
		print("Skill Helper Minimap Icon |cff00ff00visible|r!")
	else
		icon:Hide("SkillHelper")
		shDB.hide = true
		print("Skill Helper Minimap Icon |cffff0000hidden|r!")
	end
end

function shMiniMap()
	local ldb = LibStub("LibDataBroker-1.1", true)
	if not ldb then return end
	local shLDB = ldb:NewDataObject("SkillHelper", {
		type = "launcher",
		icon = "Interface\\AddOns\\SkillHelper\\IMAGES\\icon",
		OnClick = function(clickedframe, button)
			if button == "RightButton" then
				if IsShiftKeyDown() then
					shMMToggle()
				else
					shLocker()
				end
			elseif button == "MiddleButton" then
				shReset()
			elseif button == "LeftButton" then
				if IsShiftKeyDown() then
					shLinks()
				else
					shToggle()
				end
			end
		end,
		OnTooltipShow = function(tt)
			tt:AddLine(Title .. " " .. Core .. "." .. Revision .. "." .. Build)
			tt:AddDoubleLine("-------------------------", " ")
			tt:AddLine("Author: " .. Author .. " / Maintainer: " .. Maintainer)
			tt:AddLine("|r  Left-Click to toggle window")
			tt:AddLine("|r  Shift+Left-Click to toggle the Links frame")
			tt:AddLine("|r  Right-Click to toggle lock")
			tt:AddLine("|r  Shift+Right-Click to toggle the minimap icon")
			tt:AddLine("|r  Middle-Click to reset")
		end,
	})
	if shLDB then
		icon = LibStub("LibDBIcon-1.0", true)
		if not icon then return end
		icon:Register("SkillHelper", shLDB, shDB)
	end
end